<?php
/**
 * 2013 - 2021 PayPlug SAS
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0).
 * It is available through the world-wide-web at this URL:
 * https://opensource.org/licenses/osl-3.0.php
 * If you are unable to obtain it through the world-wide-web, please send an email
 * to contact@payplug.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PayPlug module to newer
 * versions in the future.
 *
 *  @author    PayPlug SAS
 *  @copyright 2013 - 2021 PayPlug SAS
 *  @license   https://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 *  International Registered Trademark & Property of PayPlug SAS
 */

class PayplugUpgraderModuleFrontController extends ModuleFrontController
{
    public function addLog($debug, $log, $str, $level)
    {
        $debugBacktrace = debug_backtrace();
        $line_n = $debugBacktrace[0]['line'];
        if ($debug) {
            $log->$level($str, '--', $line_n);
        }
        return($str);
    }

    public function postProcess()
    {
        //Inclusions
        require_once(dirname(__FILE__) . '/../../../../config/config.inc.php');
        require_once(_PS_MODULE_DIR_ . '../init.php');
        require_once(_PS_MODULE_DIR_ . 'payplug/payplug.php');
        require_once(_PS_MODULE_DIR_ . 'payplug/classes/PayplugBackward.php');
        require_once(_PS_MODULE_DIR_ . 'payplug/lib/init.php');

        //Settings
        $payplug = new Payplug();
        $debug = $payplug->getConfiguration('PAYPLUG_DEBUG_MODE');

        if ($debug) {
            require_once(dirname(__FILE__) . '/../../classes/MyLogPHP.class.php');
            $log = new MyLogPHP(_PS_MODULE_DIR_ . 'payplug/log/upgrader-' . date("Y-m-d") . '.csv');
            $this->addLog($debug, $log, '---------------- NEW UPGRADE REQUEST RECEIVED ----------------', 'info');
        } else {
            $log = false;
        }

        if ((int)Tools::getValue('debug') == 1) {
            $cid = $payplug->getConfiguration('PAYPLUG_COMPANY_ID');
            if (Tools::getValue('cid') == $cid) {
                if (($up_version = Tools::getValue('up')) !== null) {
                    require_once(dirname(__FILE__) . '/../../upgrade/upgrade-' . $up_version . '.php');
                    $up_version = str_replace('.', '_', $up_version);
                    $method = 'upgrade_module_' . $up_version;
                    $upgrade = $method($payplug);
                }
                $this->addLog($debug, $log, 'OK', 'info');
                die(Tools::jsonEncode($upgrade));
            } else {
                header(
                    $_SERVER['SERVER_PROTOCOL'] . ' 503 Access not granted.',
                    true,
                    503
                );
                die(Tools::jsonEncode($this->addLog($debug, $log, 'KO: Access not granted.', 'error')));
            }
        } else {
            header(
                $_SERVER['SERVER_PROTOCOL'] . ' 503 Access not granted.',
                true,
                503
            );
            die(Tools::jsonEncode($this->addLog($debug, $log, 'KO: Access not granted.', 'error')));
        }
    }
}
